/* In this version modular_03.c we divide the calculation in functions
so that we can perform the calculation in different ways, depending
on the functions we call.

   We perform the integration of the trajectories of a group of rotators:
   Numerical simulation of a system of coupled rotators
   in a microcanonical, symplectic integration (Yoshida,
   Phys. Lett. A 150 (1990) 262-268).

In each realization we do an estimation of the probability of the microstate p_i,
and then we average over many realizations*/


/*
  PROGRAM STRUCTURE:

  (a) Variable declaration

  (b) Variable initialization

  (c) Memory allocation

  (d) Initialization of the integration constants

  for(vez=1; vez<=VEZ; vez++){

    (e) Variable initialization

    (f) Initial values of momenta and coordinates

    (g) Transient: for(t=1;t<=Ttrans;t++){Dynamics}

    for(t=1; t<=T; t++){

      (h) Dynamics

      (i) Calculation of the cell coordinates and building of the
      occupation numbers occup_index

      if(t==T){

        (j) Calculation of the cell energy and building of the
	   occupation numbers occup_index

	(k) Averages calculation and file printing

      }

      (l) Calculation of dynamical average energies and ouput in KM+ERR...

    }

  }

*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <unistd.h>

/*(a) EXTERNAL VARIABLES DEFINITION*/

/*Variables used by many functions*/
int N, M, VEZ, vez;
unsigned long T, t, Ttrans;
double U;

/*Variables firstly used by initial_mom_coor() and/or dynamics() and/or dynamical_averages*/
double *q, *p, *F;
double m_target, mx, my, mx_M, my_M, V, VM, Vi, K, KM, tau;
double sigma_gauss_q, sigma_gauss_p;

/*Variables firstly used by make_occup_index and/or make_E_index*/
int E_dim;
int *occup_index, *occup_index_histo, *occup_K_index, *occup_K_index_histo;
double DE;
double *E_index;

/*Variables firstly used by averages_and_print*/
int *real_index, *real_K_index;
double *P_energy, *PP_energy, *P_energy_histo;
double *P_K_energy, *P_K_energy_histo;
char file_name[100], file_name_K[100], file_name1[100];
time_t tmpi, tmpf;

/*Variables firstly used by dynamical_averages*/
int size_dat=1000, i, cont=0, contint=0, prima=1;
double **dat, *EK_med, *VM_med,  *Vi_med, *m_med;
char file_name2[100];

/*Function declaration*/
void initial_mom_coor();
void dynamics_initialization();
void dynamics(double tau);
void make_occup_index();
void make_E_index();
void averages_and_print();
void dynamical_averages(unsigned long t);

int main(int argc, char **argv){

  double S2,E_max;

  /* initial time (to calculate elpased time)*/
  tmpi=time(&tmpi);

  /*(b) Global variable initialization*/
  N   		= atoi(argv[1]);
  M   		= atoi(argv[2]);
  tau 		= atof(argv[3]);
  U   		= atof(argv[4]);
  VEZ 		= atoi(argv[5]);
  S2		= atof(argv[6]);
  m_target	= atof(argv[7]);
  Ttrans	= atoi(argv[8]);
  T		= atoi(argv[9]);
  E_max		= atof(argv[10]);
  sigma_gauss_q	= atof(argv[11]);
  sigma_gauss_p	= atof(argv[12]);

  if(argv[13]==NULL)   argv[13]="";

  /*Some definitions*/
  DE=1./(double)S2;
  E_dim=(int)(E_max/DE);

  /*Some printouts for checking that the parameters are ok*/
  printf("DE=%lf\n",DE);
  printf("E_min=%lf, M=%d, E_dim=%d\n", 2*M*DE/2, M, E_dim);
  printf("PID-->%d\n\nN=%d\nM=%d\nDE=%lf\n\n\n",getpid(),N,M,DE);

  /*Contruction of filenames*/
  sprintf(file_name,  "o_E-U%.2lfN%d-M%d-%d%s.dat",    U, N, M, getpid(), argv[13]);
  sprintf(file_name1, "info-U%.2lfN%d-M%d-%d%s.dat",   U, N, M, getpid(), argv[13]);
  sprintf(file_name2, "KM+err-U%.2lfN%d-M%d-%d%s.dat", U, N, M, getpid(), argv[13]);
  sprintf(file_name_K,"o_K-U%.2lfN%d-M%d-%d%s.dat",    U, N, M, getpid(), argv[13]);

  /*(c) Memory allocation*/
  if((q=(double *)calloc(N+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((p=(double *)calloc(N+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((F=(double *)calloc(N+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((occup_index=(int *)calloc(E_dim+1, sizeof(int)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((occup_index_histo=(int *)calloc(E_dim+1, sizeof(int)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((occup_K_index=(int *)calloc(E_dim+1, sizeof(int)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((occup_K_index_histo=(int *)calloc(E_dim+1, sizeof(int)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((E_index=(double *)calloc(E_dim+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((real_index=(int *)calloc(E_dim+1, sizeof(int)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((real_K_index=(int *)calloc(E_dim+1, sizeof(int)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((P_energy=(double *)calloc(E_dim+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((PP_energy=(double *)calloc(E_dim+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((P_energy_histo=(double *)calloc(E_dim+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((P_K_energy=(double *)calloc(E_dim+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((P_K_energy_histo=(double *)calloc(E_dim+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((dat=(double **)calloc(size_dat+1, sizeof(double *)))==NULL)
    {printf("err no mem\n");exit(1);}
  for(i=1; i<=size_dat; i++)
    if((*(dat+i)=(double *)calloc(5, sizeof(double)))==NULL)
      {printf("err no mem\n");exit(1);}

  if((EK_med=(double *)calloc(T+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((VM_med=(double *)calloc(T+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((Vi_med=(double *)calloc(T+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  if((m_med=(double *)calloc(T+1, sizeof(double)))==NULL)
    {printf("err no mem\n");exit(1);}

  /*(d) Initialization of the integration constants*/
  dynamics_initialization();

  for(vez=1; vez<=VEZ; vez++){

    /*(e) Variable initialization (for the KM+ERR file)*/
    prima=1;
    contint=0;

    /*(f) Initial values of momenta and coordinates*/
    initial_mom_coor();

    /*(g) Transient*/
    for(t=1;t<=Ttrans;t++) dynamics(tau);

    for(t=1; t<=T; t++){
      /*(g) Dynamics*/
      dynamics(tau);

      /*(i) Calculation of the cell coordinates and building of the
      occupation numbers occup_index*/
      make_occup_index();

      if(t==T){
	/*(j) Calculation of the cell energy and building of the
	   occupation numbers occup_index*/
	make_E_index();
	/*(k) Averages calculation and file printing*/
	averages_and_print();
      }

      /*(l) Calculation of dynamical average energies and ouput in KM+ERR...*/
      dynamical_averages(t);
    }
  }

 /*Elapsed time*/
 tmpf=time(&tmpf);
 printf("\nElapsed time: %.1lf s\n", difftime(tmpf,tmpi));

 return 0;

}



